/*
 *	FGA - Fast Genetic Algorithm
 *	Copyright (C) 2007 Alessandro Presta (alessandro.presta@gmail.com)
 *	All rights reserved.
 *
 *	This library is free for commercial and non-commercial use as long as
 *	the following conditions are aheared to. The following conditions 
 *	apply to all code found in this distribution, be it the library itself
 *	or the included examples. All the documentation included with this 
 *	distribution is covered by the same copyright terms.
 *	
 *	Copyright remains Alessandro Presta's, and as such any Copyright 
 *	notices in the code are not to be removed.
 *	If this package is used in a product, Alessandro Presta should be 
 *	given attribution as the author of the parts of the library used.
 *	This can be in the form of a textual message at program startup or
 *	in documentation (online or textual) provided with the package.
 *
 *	Redistribution and use in source and binary forms, with or without
 *	modification, are permitted provided that the following conditions
 *	are met:
 *	1. Redistributions of source code must retain the copyright
 *	   notice, this list of conditions and the following disclaimer.
 *	2. Redistributions in binary form must reproduce the above copyright
 *	   notice, this list of conditions and the following disclaimer in the
 *	   documentation and/or other materials provided with the distribution.
 *	3. All advertising materials mentioning features or use of this software
 *	   must display the following acknowledgement:
 *	   "This product includes the Fast Genetic Algorithm library written by
 *	   Alessandro Presta (alessandro.presta@gmail.com)"
 *
 * 	THIS SOFTWARE IS PROVIDED BY ALESSANDRO PRESTA ``AS IS'' AND
 * 	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * 	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * 	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * 	LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * 	OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * 	SUCH DAMAGE.
 * 
 * 	The licence and distribution terms for any publically available version or
 * 	derivative of this code cannot be changed.  i.e. this code cannot simply be
 * 	copied and put under another distribution licence
 *	(including the GNU Public Licence).
 */

#ifndef FGA_HPP_
#define FGA_HPP_

#include <algorithm>
#include <cmath>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <map>
#include <pthread.h>

using namespace std;

namespace fga {
	
	enum {
		CROSSOVER_1POINT,
		CROSSOVER_2POINT,
		CROSSOVER_UNIFORM,
		CROSSOVER_CUSTOM
	};
	
	enum {
		SELECT_WEIGHTED,
		SELECT_UNIFORM,
		SELECT_TOURNAMENT,
		SELECT_ELITIST,
		SELECT_CUSTOM
	};

	enum {
		MUTATE_DEFAULT,
		MUTATE_CUSTOM
	};
	
	enum {
		RANDOM_DEFAULT,
		RANDOM_CUSTOM
	};
	
	enum {
		TOPOLOGY_COMPLETE,
		TOPOLOGY_RING,
		TOPOLOGY_BIDIRECTIONAL_RING,
		TOPOLOGY_ISOLATED
	};

	template <class GeneType>
	class Population {
		private:
			int n;
			GeneType **chromosome, **child_chromosome;
			int length;
			float *score, *child_score;
			float total_score, child_total_score;
			int best, child_best;
			int generations;
			float crossover_rate, mutation_rate;
			int crossover_type, select_type, mutate_type, random_type;
			int tournament_size, elite_size, elite_left;
			int *order_by_fitness;
			bool preserve_best;
			float (*fitness)(GeneType *);
			GeneType (*mutate_gene)(GeneType);
			GeneType (*random_gene)();
			void (*crossover_custom)(GeneType *, GeneType *);
			void (*mutate_custom)(GeneType *);
			void (*random_custom)(GeneType *);
			GeneType *(*select_custom)(GeneType **);
			GeneType *select();
			GeneType *select_uniform();
			GeneType *select_tournament();
			GeneType *select_elitist();
			void crossover(GeneType *c1, GeneType *c2);
			void crossover_2point(GeneType *c1, GeneType *c2);
			void crossover_uniform(GeneType *c1, GeneType *c2);
			void mutate(GeneType *c);
			struct compare {
				float *value;
				compare(float *val);
				bool operator()(int c1, int c2);
			};
		public:
			Population(int nc, int clength, float (*f)(GeneType *), GeneType (*mgene)(GeneType), GeneType (*rgene)(), void (*ccustom)(GeneType *, GeneType *), void (*mcustom)(GeneType *), void (*rcustom)(GeneType *), GeneType *(*scustom)(GeneType **));
			~Population();
			void set_crossover_rate(float crate);
			void set_mutation_rate(float mrate);
			void set_crossover_type(int ctype);
			void set_select_type(int stype);
			void set_tournament_size(int tsize);
			void set_elite_size(int esize);
			void set_preserve_best(bool p);
			GeneType *get_best();
			float get_best_score();
			float get_avg_score();
			float get_standard_deviation();
			GeneType *get_chromosome(int c);
			float get_score(int c);
			int get_generations();
			bool has_converged();
			void migrate(GeneType *c);
			bool run(float min_tolerated_fitness, int max_generations);
			bool run(int stable_generations);
			bool run();
			void cycle();
			void save(char *file);
			void save(ofstream& out);
			void load(char *file);
			void load(ifstream& in);
	};
	
	template <class GeneType>
	class PopulationMT {
		private:
			int nthreads;
			int n;
			int length;
			Population<GeneType> **tribe;
			pthread_t *thread;
			int best_tribe;
			int migration_interval, migration_count, migration_size;
			int topology;
			static void *thread_wrapper(void *t);
			void perform_migration();
			void perform_migration_complete();
			void perform_migration_ring();
			void perform_migration_bidirectional_ring();
		public:
			PopulationMT(int nt, int nc, int clength, float (*f)(GeneType *), GeneType (*mgene)(GeneType), GeneType (*rgene)(), void (*ccustom)(GeneType *, GeneType *), void (*mcustom)(GeneType *), void (*rcustom)(GeneType *), GeneType *(*scustom)(GeneType **));
			~PopulationMT();
			void set_crossover_rate(float crate);
			void set_mutation_rate(float mrate);
			void set_crossover_type(int ctype);
			void set_select_type(int stype);
			void set_tournament_size(int tsize);
			void set_elite_size(int esize);
			void set_migration_interval(int mint);
			void set_migration_size(int msize);
			void set_topology(int t);
			void set_preserve_best(bool p);
			GeneType *get_best();
			float get_best_score();
			float get_avg_score();
			float get_standard_deviation();
			GeneType *get_chromosome(int c);
			float get_score(int c);
			int get_generations();
			bool has_converged();
			bool run(float min_tolerated_fitness, int max_generations);
			bool run(int stable_generations);
			bool run();
			void cycle();
			void save(char *file);
			void save(ofstream& out);
			void load(char *file);
			void load(ifstream& in);
	};
	
	template <class GeneType>
	Population<GeneType>::Population(int nc, int clength, float (*f)(GeneType *), GeneType (*mgene)(GeneType), GeneType (*rgene)(), void (*ccustom)(GeneType *, GeneType *), void (*mcustom)(GeneType *), void (*rcustom)(GeneType *), GeneType *(*scustom)(GeneType **))
	{
		n = nc;
		length = clength;
		fitness = f;
		mutate_gene = mgene;
		random_gene = rgene;
		crossover_custom = ccustom;
		mutate_custom = mcustom;
		random_custom = rcustom;
		select_custom = scustom;
		crossover_rate = 0.7;
		mutation_rate = 0.001;
		crossover_type = CROSSOVER_1POINT;
		select_type = SELECT_WEIGHTED;
		mutate_type = MUTATE_DEFAULT;
		random_type = RANDOM_DEFAULT;
		tournament_size = 2;
		elite_size = n / 10;
		if (crossover_custom != NULL)
			crossover_type = CROSSOVER_CUSTOM;
		if (mutate_custom != NULL)
			mutate_type = MUTATE_CUSTOM;		
		if (random_custom != NULL)
			random_type = RANDOM_CUSTOM;
		if (select_custom != NULL)
			select_type = SELECT_CUSTOM;
		preserve_best = true;		
		chromosome = new GeneType*[n];
		child_chromosome = new GeneType*[n];
		score = new float[n];
		child_score = new float[n];
		order_by_fitness = new int[n];
		total_score = 0;
		best = 0;
		generations = 0;
		for (int i = 0; i < n; ++i) {
			chromosome[i] = new GeneType[length];
			child_chromosome[i] = new GeneType[length];
			if (random_type == RANDOM_DEFAULT) {			
				for (int j = 0; j < length; ++j)
					chromosome[i][j] = random_gene();
			}
			else
				random_custom(chromosome[i]);
			score[i] = fitness(chromosome[i]);
			total_score += score[i];
			if (score[i] > score[best])
				best = i;
		}
	}
	
	template <class GeneType>
	Population<GeneType>::~Population()
	{
		for (int i = 0; i < n; ++i) {
			delete[] chromosome[i];
			delete[] child_chromosome[i];
		}
		delete[] chromosome;
		delete[] child_chromosome;
		delete[] score;
		delete[] child_score;
		delete[] order_by_fitness;
	}
	
	template <class GeneType>
	void Population<GeneType>::set_crossover_rate(float crate)
	{
		crossover_rate = crate;
	}

	template <class GeneType>		
	void Population<GeneType>::set_mutation_rate(float mrate)
	{
		mutation_rate = mrate;
	}
	
	template <class GeneType>
	void Population<GeneType>::set_crossover_type(int ctype)
	{
		crossover_type = ctype;
	}
			
	template <class GeneType>
	void Population<GeneType>::set_select_type(int stype)
	{
		select_type = stype;
	}
	
	template <class GeneType>
	void Population<GeneType>::set_tournament_size(int tsize)
	{
		tournament_size = tsize;
	}
	
	template <class GeneType>
	void Population<GeneType>::set_elite_size(int esize)
	{
		elite_size = esize;
	}

	template <class GeneType>
	void Population<GeneType>::set_preserve_best(bool p)
	{
		preserve_best = p;
	}
	
	template <class GeneType>
	GeneType *Population<GeneType>::get_best()
	{
		return chromosome[best];
	}
	
	template <class GeneType>
	float Population<GeneType>::get_best_score()
	{
		return score[best];
	}

	template <class GeneType>
	float Population<GeneType>::get_avg_score()
	{
		return total_score / n;	
	}

	template <class GeneType>
	float Population<GeneType>::get_standard_deviation()
	{
		float variance = 0;
		float average = get_avg_score();
		for (int i = 0; i < n; ++i)
			variance += pow(score[i] - average, 2);
		return sqrt(variance) / n;
	}

	template <class GeneType>
	GeneType *Population<GeneType>::get_chromosome(int c)
	{
		return chromosome[c];
	}
	
	template <class GeneType>
	float Population<GeneType>::get_score(int c)
	{
		return score[c];
	}

	template <class GeneType>
	int Population<GeneType>::get_generations()
	{
		return generations;
	}
	
	template <class GeneType>
	bool Population<GeneType>::has_converged()
	{
		map<GeneType, int> *equal_genes = new map<GeneType, int>[length];
		int *max_equal = new int[length];
		memset(max_equal, 0, length * sizeof(int));
		bool converged = true;
		for (int i = 0; i < n; ++i)
			for (int j = 0; j < length; ++j)
				if (equal_genes[j].find(chromosome[i][j]) == equal_genes[j].end()) {
					equal_genes[j][chromosome[i][j]] = 1;
					if (max_equal[j] == 0)
						max_equal[j] = 1;
				}
				else {
					++equal_genes[j][chromosome[i][j]];
					if (equal_genes[j][chromosome[i][j]] > max_equal[j])
						max_equal[j] = equal_genes[j][chromosome[i][j]];
				}
		for (int i = 0; i < length; ++i)
			if ((float)max_equal[i] / n < 0.95) {
				converged = false;
				break;
			}
		delete[] equal_genes;
		delete[] max_equal;
		return converged;
	}
	
	template <class GeneType>
	void Population<GeneType>::migrate(GeneType *c)
	{
		int r;
		do
			r = rand() % n;
		while (score[r] > get_avg_score());
		total_score -= score[r];
		memcpy(chromosome[r], c, length * sizeof(GeneType));
		score[r] = fitness(chromosome[r]);
		total_score += score[r];
	}
	
	template <class GeneType>
	bool Population<GeneType>::run(float min_tolerated_fitness, int max_generations)
	{
		for (int i = 0; i < max_generations; ++i) {
			if (get_best_score() >= min_tolerated_fitness)
				return true;
			cycle();
		}
		return false;
	}
	
	template <class GeneType>
	bool Population<GeneType>::run(int stable_generations)
	{
		int generations_count = 0;
		float best_fitness = -1;
		do {
			cycle();
			if (get_best_score() > best_fitness) {
				best_fitness = get_best_score();
				generations_count = 1;
			}
			else
				++generations_count;
		} while (generations_count < stable_generations);
		return true;
	}
	
	template <class GeneType>
	bool Population<GeneType>::run()
	{
		while (!has_converged())
			cycle();
		return true;
	}
	
	template <class GeneType>
	void Population<GeneType>::cycle()
	{
		if (select_type == SELECT_ELITIST) {
			elite_left = elite_size;
			for (int i = 0; i < n; ++i)
				order_by_fitness[i] = i;
			sort(order_by_fitness, order_by_fitness + n, compare(score));
		}
		child_total_score = 0;
		child_best = 0;
		for (int i = 0; i < (n / 2) * 2; i += 2) {
			GeneType *parent0 = select();
			GeneType *parent1 = select();
			GeneType *child0 = child_chromosome[i];
			GeneType *child1 = child_chromosome[i + 1];
			memcpy(child0, parent0, length * sizeof(GeneType));
			memcpy(child1, parent1, length * sizeof(GeneType));	
			crossover(child0, child1);
			mutate(child0);
			mutate(child1);
			child_score[i] = fitness(child0);
			child_score[i + 1] = fitness(child1);
			child_total_score += child_score[i] + child_score[i + 1];
			if (child_score[i] > child_score[child_best])
				child_best = i;
			if (child_score[i + 1] > child_score[child_best])
				child_best = i + 1;
		}
		if (n % 2 == 1) {
			memcpy(child_chromosome[n - 1], select(), length * sizeof(GeneType));
			child_score[n - 1] = fitness(child_chromosome[n - 1]);
			child_total_score += child_score[n - 1];
			if (child_score[n - 1] > child_score[child_best])
				child_best = n - 1;
		}
		if (preserve_best && child_score[child_best] < score[best]) {
			int r;
			do
				r = rand() % n;
			while (child_score[r] > child_total_score / n);
			child_total_score -= child_score[r];
			memcpy(child_chromosome[r], chromosome[best], length * sizeof(GeneType));
			child_score[r] = score[best];
			child_total_score += child_score[r];
			child_best = r;
		}
		GeneType **tmp_chromosome = chromosome;
		chromosome = child_chromosome;
		child_chromosome = tmp_chromosome;
		float *tmp_score = score;
		score = child_score;
		child_score = tmp_score;
		total_score = child_total_score;
		best = child_best;
		++generations;
	}
	
	template <class GeneType>
	void Population<GeneType>::save(char *file)
	{
		ofstream out(file);
		save(out);
		out.close();
	}
	
	template <class GeneType>
	void Population<GeneType>::save(ofstream& out)
	{
		out << "FGA ST" << endl;
		out << generations << endl;
		for (int i = 0; i < n; ++i) {
			for (int j = 0; j < length; ++j)
				out << chromosome[i][j] << " ";
			out << endl;
		}
		out << crossover_rate << endl;
		out << mutation_rate << endl;
		out << crossover_type << endl;
		out << select_type << endl;
		out << mutate_type << endl;
		out << random_type << endl;
		out << tournament_size << endl;
		out << elite_size << endl;
	}
	
	template <class GeneType>
	void Population<GeneType>::load(char *file)
	{
		ifstream in(file);
		load(in);
		in.close();
	}
	
	template <class GeneType>
	void Population<GeneType>::load(ifstream& in)
	{
		char str[4];
		in >> str >> str;
		in >> generations;
		for (int i = 0; i < n; ++i)
			for (int j = 0; j < length; ++j)
				in >> chromosome[i][j];
		in >> crossover_rate;
		in >> mutation_rate;
		in >> crossover_type;
		in >> select_type;
		in >> mutate_type;
		in >> random_type;
		in >> tournament_size;
		in >> elite_size;
		total_score = 0;
		best = 0;
		for (int i = 0; i < n; ++i) {
			score[i] = fitness(chromosome[i]);
			total_score += score[i];
			if (score[i] > score[best])
				best = i;
		}
	}
			
	template <class GeneType>		
	inline GeneType *Population<GeneType>::select()
	{
		switch (select_type) {
			case SELECT_CUSTOM:
				return select_custom(chromosome);
			case SELECT_UNIFORM:
				return select_uniform();
			case SELECT_TOURNAMENT:
				return select_tournament();
			case SELECT_ELITIST:
				return select_elitist();
			default:
				float choice = (float)rand() / RAND_MAX * total_score;
				float position = 0;
				for (int i = 0; i < n; ++i) {
					position += score[i];
					if (position >= choice)
						return chromosome[i];
				}
				return chromosome[n - 1];
		}		
	}
	
	template <class GeneType>
	inline GeneType *Population<GeneType>::select_uniform()
	{
		return chromosome[rand() % n];
	}

	template <class GeneType>
	inline GeneType *Population<GeneType>::select_tournament()
	{
		int best_candidate = -1;
		float best_fitness = -1;
		if (tournament_size <= 0)
			return chromosome[rand() % n];
		bool *chosen = new bool[n];
		memset(chosen, 0, n * sizeof(bool));
		for (int i = 0; i < tournament_size; ++i) {
			int r = rand() % n;
			while (chosen[r])
				r = (r + 1) % n;
			if (score[r] > best_fitness) {
				best_fitness = score[r];
				best_candidate = r;	
			}
			chosen[r] = true;
		}
		delete[] chosen;
		return chromosome[best_candidate];
	}

	template <class GeneType>
	inline GeneType *Population<GeneType>::select_elitist()
	{
		if (elite_left > 0)
			return (chromosome[order_by_fitness[elite_size - elite_left--]]);
		else
			return select_uniform();
	}
	
	template <class GeneType>
	inline void Population<GeneType>::crossover(GeneType *c1, GeneType *c2)
	{
		if (((float)rand() / RAND_MAX) <= crossover_rate) {
			switch (crossover_type) {
				case CROSSOVER_2POINT:
					crossover_2point(c1, c2);
					break;
				case CROSSOVER_UNIFORM:
					crossover_uniform(c1, c2);
					break;
				case CROSSOVER_CUSTOM:
					crossover_custom(c1, c2);
					break;
				default:
					int cut = rand() % length;
					GeneType tmp;
					for (int i = cut; i < length; ++i) {
						tmp = c1[i];
						c1[i] = c2[i];
						c2[i] = tmp;
					}
			}
		}	
	}
	
	template <class GeneType>
	inline void Population<GeneType>::crossover_2point(GeneType *c1, GeneType *c2)
	{
		int cut = rand() % length;
		GeneType tmp;
		for (int i = 0; i < cut; ++i) {
			tmp = c1[i];
			c1[i] = c2[i];
			c2[i] = tmp;
		}
		cut += rand() % (length - cut);
		for (int i = cut; i < length; ++i) {
			tmp = c1[i];
			c1[i] = c2[i];
			c2[i] = tmp;
		}
	}
	
	template <class GeneType>
	inline void Population<GeneType>::crossover_uniform(GeneType *c1, GeneType *c2)
	{
		GeneType tmp;
		for (int i = 0; i < length; ++i)
			if (rand() <= RAND_MAX / 2) {
				tmp = c1[i];
				c1[i] = c2[i];
				c2[i] = tmp;	
			}
	}
	
	template <class GeneType>
	inline void Population<GeneType>::mutate(GeneType *c)
	{
		if (mutate_type == MUTATE_CUSTOM) {
			if ((float)rand() / RAND_MAX <= mutation_rate)
				mutate_custom(c);
			return;
		}
		for (int i = 0; i < length; ++i)
			if ((float)rand() / RAND_MAX <= mutation_rate)
				c[i] = mutate_gene(c[i]);
	}
	
	template <class GeneType>
	Population<GeneType>::compare::compare(float *val)
	{
		value = val;
	}
	
	template <class GeneType>
	inline bool Population<GeneType>::compare::operator()(int c1, int c2)
	{
		return (value[c1] > value[c2]);
	}
	
	template <class GeneType>
	PopulationMT<GeneType>::PopulationMT(int nt, int nc, int clength, float (*f)(GeneType *), GeneType (*mgene)(GeneType), GeneType (*rgene)(), void (*ccustom)(GeneType *, GeneType *), void (*mcustom)(GeneType *), void (*rcustom)(GeneType *), GeneType *(*scustom)(GeneType **))
	{
		nthreads = max(nt, 2);
		n = nc;
		length = clength;
		tribe = new Population<GeneType>*[nthreads]; 
		thread = new pthread_t[nthreads];
		for (int i = 0; i < nthreads; ++i)
			tribe[i] = new Population<GeneType>(n, clength, f, mgene, rgene, ccustom, mcustom, rcustom, scustom);	
		best_tribe = 0;
		for (int i = 0; i < nthreads; ++i)			
			if (tribe[i]->get_best_score() > get_best_score())
				best_tribe = i;
		migration_interval = 10;
		migration_count = 0;
		migration_size = n / (10 * (nthreads - 1));
		topology = TOPOLOGY_COMPLETE;
	}
	
	template <class GeneType>
	PopulationMT<GeneType>::~PopulationMT()
	{
		for (int i = 0; i < nthreads; ++i)
			delete tribe[i];
		delete[] tribe;
		delete[] thread;
	}
		
	template <class GeneType>
	void PopulationMT<GeneType>::set_crossover_rate(float crate)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_crossover_rate(crate);
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::set_mutation_rate(float mrate)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_mutation_rate(mrate);
	}
		
	template <class GeneType>
	void PopulationMT<GeneType>::set_crossover_type(int ctype)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_crossover_type(ctype);
	}
		
	template <class GeneType>
	void PopulationMT<GeneType>::set_select_type(int stype)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_select_type(stype);
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::set_tournament_size(int tsize)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_tournament_size(tsize);
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::set_elite_size(int esize)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_elite_size(esize);
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::set_migration_interval(int mint)
	{
		migration_interval = mint;	
	}
		
	template <class GeneType>
	void PopulationMT<GeneType>::set_migration_size(int msize)
	{
		migration_size = msize;	
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::set_topology(int t)
	{
		topology = t;
	}

	template <class GeneType>
	void PopulationMT<GeneType>::set_preserve_best(bool p)
	{
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->set_preserve_best(p);
	}
	
	template <class GeneType>
	GeneType *PopulationMT<GeneType>::get_best()
	{
		return tribe[best_tribe]->get_best();
	}
		
	template <class GeneType>
	float PopulationMT<GeneType>::get_best_score()
	{
		return tribe[best_tribe]->get_best_score();
	}

	template <class GeneType>
	float PopulationMT<GeneType>::get_avg_score()
	{
		float sum = 0;
		for (int i = 0; i < nthreads; ++i)
			sum += tribe[i]->get_avg_score();
		return sum / nthreads;
	}
	
	template <class GeneType>
	float PopulationMT<GeneType>::get_standard_deviation()
	{
		float variance = 0;
		float average = get_avg_score();
		for (int i = 0; i < nthreads; ++i)
			for (int j = 0; j < n; ++j)
				variance += pow(tribe[i]->get_score(j) - average, 2);
		return sqrt(variance) / (n * nthreads);
	}
	
	template <class GeneType>
	GeneType *PopulationMT<GeneType>::get_chromosome(int c)
	{
		return tribe[c / n]->get_chromosome(c % n);
	}
	
	template <class GeneType>
	float PopulationMT<GeneType>::get_score(int c)
	{
		return tribe[c / n]->get_score(c % n);
	}
		
	template <class GeneType>
	int PopulationMT<GeneType>::get_generations()
	{
		return tribe[0]->get_generations();
	}
	
	template <class GeneType>
	bool PopulationMT<GeneType>::has_converged()
	{
		for (int i = 0; i < nthreads; ++i)
			if (!tribe[i]->has_converged())
				return false;
		return true;
	}
		
	template <class GeneType>
	bool PopulationMT<GeneType>::run(float min_tolerated_fitness, int max_generations)
	{
		for (int i = 0; i < max_generations; ++i) {
			if (get_best_score() >= min_tolerated_fitness)
				return true;
			cycle();
		}
		return false;
	}
	
	template <class GeneType>
	bool PopulationMT<GeneType>::run(int stable_generations)
	{
		int generations_count = 0;
		float best_fitness = -1;
		do {
			cycle();
			if (get_best_score() > best_fitness) {
				best_fitness = get_best_score();
				generations_count = 1;
			}
			else
				++generations_count;
		} while (generations_count < stable_generations);
		return true;
	}
	
	template <class GeneType>
	bool PopulationMT<GeneType>::run()
	{
		while (!has_converged())
			cycle();
		return true;
	}
		
	template <class GeneType>
	void PopulationMT<GeneType>::cycle()
	{
		for (int i = 0; i < nthreads; ++i)
			pthread_create(&(thread[i]), NULL, thread_wrapper, (void *)&(*(tribe[i])));
		for (int i = 0; i < nthreads; ++i)
			pthread_join(thread[i], NULL);
		best_tribe = 0;
		for (int i = 0; i < nthreads; ++i)
			if (tribe[i]->get_best_score() > get_best_score())
				best_tribe = i;
		if (++migration_count == migration_interval) {
			migration_count = 0;
			perform_migration();
		}
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::save(char *file)
	{
		ofstream out(file);
		save(out);
		out.close();
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::save(ofstream& out)
	{
		out << "FGA MT" << endl;
		out << migration_interval << endl;
		out << migration_size << endl;
		out << topology << endl;
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->save(out);
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::load(char *file)
	{
		ifstream in(file);
		load(in);
		in.close();
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::load(ifstream& in)
	{
		char str[4];
		in >> str >> str;
		in >> migration_interval;
		in >> migration_size;
		in >> topology;
		for (int i = 0; i < nthreads; ++i)
			tribe[i]->load(in);
		best_tribe = 0;
		for (int i = 0; i < nthreads; ++i)
			if (tribe[i]->get_best_score() > get_best_score())
				best_tribe = i;
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::perform_migration()
	{
		switch (topology) {
			case TOPOLOGY_COMPLETE:
				perform_migration_complete();
				break;
			case TOPOLOGY_RING:
				perform_migration_ring();
				break;
			case TOPOLOGY_BIDIRECTIONAL_RING:
				perform_migration_bidirectional_ring();
		}
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::perform_migration_complete()
	{
		for (int i = 0; i < nthreads - 1; ++i)
			for (int j = i + 1; j < nthreads; ++j)
				for (int k = 0; k < migration_size; ++k) {
					int r = rand() % n;
					if (tribe[i]->get_score(r) >= tribe[j]->get_avg_score())
						tribe[j]->migrate(tribe[i]->get_chromosome(r));
					r = rand() % n;
					if (tribe[j]->get_score(r) >= tribe[i]->get_avg_score())
						tribe[i]->migrate(tribe[j]->get_chromosome(r));
				}
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::perform_migration_ring()
	{
		for (int i = 0; i < nthreads; ++i)
			for (int k = 0; k < migration_size; ++k) {
				int r = rand() % n;
				int j = (i + 1) % nthreads;
				if (tribe[i]->get_score(r) >= tribe[j]->get_avg_score())
					tribe[j]->migrate(tribe[i]->get_chromosome(r));
			}
	}
	
	template <class GeneType>
	void PopulationMT<GeneType>::perform_migration_bidirectional_ring()
	{
		for (int i = 0; i < nthreads; ++i)
			for (int k = 0; k < migration_size; ++k) {
				int r = rand() % n;
				int j = (i + 1) % nthreads;
				if (tribe[i]->get_score(r) >= tribe[j]->get_avg_score())
					tribe[j]->migrate(tribe[i]->get_chromosome(r));
				r = rand() % n;
				if (tribe[j]->get_score(r) >= tribe[i]->get_avg_score())
					tribe[i]->migrate(tribe[j]->get_chromosome(r));
			}
	}
	
	template <class GeneType>
	void *PopulationMT<GeneType>::thread_wrapper(void *t)
	{
		((Population<GeneType> *)t)->cycle();
		return NULL;
	}
	
}

#endif // FGA_HPP_

